﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MCScheduling.MixedCriticality ;
using System.Windows.Forms.DataVisualization.Charting ;

namespace MCScheduling
{
    /**
     * The GanttViewForm is a window whose client are is filled
     * with a char area that display a Gant diagram of an MC
     * instance.
     */
    public partial class GanttViewForm : Form
    {
        /**
         * Creates a window that display Gant diagram of the specified
         * MC instance.
         * 
         * @param instance The MC instance to be display in the Gant diagram.
         */
        public GanttViewForm(CMixedCriticalityInstance instance)
        {
            InitializeComponent() ;
            loadSchedule(instance) ;
        }

        /**
         * Loads the specified MC instance into the chart area of
         * the GanttView window.
         * 
         * @param instance The MC instance to be display in the Gant diagram.
         */
        private void loadSchedule(CMixedCriticalityInstance instance)
        {
            /* Checks the validity of the instance. */
            foreach (CMixedCriticalityJob job in instance)
            {
                if (double.IsInfinity(job.getStartTime()) || double.IsNaN(job.getStartTime()))
                {
                    throw new InvalidOperationException() ;
                }
            }

            Series series = new Series() ;

            series.ChartArea       = "ganttViewArea" ;
            series.Name            = instance.getName() ;
            series.BorderColor     = System.Drawing.Color.Black ;
            series.ChartType       = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.RangeBar ;
            series.YAxisType       = AxisType.Secondary ;
            series.YValuesPerPoint = 2 ;

            series.SmartLabelStyle.Enabled = false ;

            Random random = new Random((int)(new DateTime()).Ticks) ;

            foreach (CMixedCriticalityJob job in instance)
            {
                Color color = Color.FromArgb(random.Next(256),random.Next(256),random.Next(256)) ;

                for (int criticality = 1 ; criticality <= job.getCriticality() ; criticality++)
                {
                    DataPoint dataPoint = new DataPoint(criticality, new double[] { job.getStartTime(), job.getCompletionTime(criticality) }) ;
                    
                    dataPoint.Label = job.getName() ;
                    dataPoint.Color = color ;

                    dataPoint.BorderColor = System.Drawing.Color.Black ;
                    dataPoint.BorderWidth = 1 ;

                    series.Points.Add(dataPoint) ;
                }
            }

            ganttViewChart.Series.Add(series) ;

            ganttViewChart.Update() ;
        }
    }
}
