﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MCScheduling.GeneticAlgorithm
{
    /**
     * A breeder evolves a population by performing genetic operations
     * defined in the specified configuration of the genetic algorithm.
     *
     * @author Tomas Tunys
     */
    public abstract class CBaseBreeder
    {
        /**
         * A configuration of a genetic algorithm that uses this 
         * breeder. 
         */
        private CConfiguration m_configuration ;

        /**
         * Creates and initializes the base breeder.
         * 
         * @param configuration The configuration of the genetic algorithm
         *                      that will utilize this breeder.
         *
         * @throws NullReferenceException If confuguration is <em>null</em>.
         */
        public CBaseBreeder(CConfiguration configuration)
        {
            if (configuration == null) 
            {
                throw new NullReferenceException("The configuration must not be null!") ;
            }
            else
            {
                m_configuration = configuration ;
            }
        }

        /**
         * Returns a configuration of a genetic algorithm
         * that uses this breeder.
         * 
         * @return A configuration of a genetic algorithm
         *         that uses this breeder.
         */
        public CConfiguration getConfiguration()
        {
            return m_configuration ;
        }

        /**
         * Evolves the specified population of chromosomes, by execution of
         * all genetic operators: selectors, crossover, and mutation operators 
         * that are currently set up in the configuration of the algorithm
         * that has been specified during initialization of the breeder.
         *
         * @param population        The population to be evolved.
         * @param evolvedPopulation A new evolved population.
         */   
        public abstract void breed(CPopulation population, CPopulation evolvedPopulation) ;
    }
}
