﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization ;
using MCScheduling.MixedCriticality.GA ;

namespace MCScheduling.GeneticAlgorithm
{
    /**
     * The serializer for the implemented finess evalutors.
     */
    class CFitnessEvaluatorSerializer
    {
        /**
         * Returns the System.String representing the serialization
         * of the specified fitness evaluator.
         * 
         * @param evaluator The scaler to be serialized.
         * 
         * @return The string representing the serialization of the 
         *         specified fitness evaluator.
         *         
         * @throws ArgumentException If scaler is null or unknown
         *                           by this serializer.
         */
        public static string serialize(IFitnessEvaluator evaluator)
        {
            if (evaluator is CMinimizeMakespanFitnessEvaluator)
            {
                CMinimizeMakespanFitnessEvaluator s = (CMinimizeMakespanFitnessEvaluator)evaluator ;

                return String.Format(CultureInfo.InvariantCulture, "MML {0:0.##} {1:0.##}", s.getMakespanFactor(), s.getLatenessFactor()) ;
            }
            else
            {
                throw new ArgumentException("Uknown or null evaluator!") ;
            }
        }

        /**
         * Creates a fitness evaluator from the specified string.
         * 
         * @param serialization The fitness evaluator serialization to be
         *                      deserialized.
         *                      
         * @return The deserialized fitness evaluator.
         *                      
         * @throws ArgumentException If the serialization is invalid,
         *                           or represents no fitness evaluator.
         */
        public static IFitnessEvaluator deserialize(string serialization)
        {
            if (serialization.StartsWith("mml", true, null))
            {
                String[] parameters = serialization.Split(' ', '\t') ;

                if (parameters.Length != 3)
                {
                    throw new ArgumentException("Minimize-Makespan-Lateness fitness evaluator parameters missing!") ;
                }

                return new CMinimizeMakespanFitnessEvaluator(double.Parse(parameters[1], CultureInfo.InvariantCulture), double.Parse(parameters[2], CultureInfo.InvariantCulture)) ;
            }
            else
            {
                throw new ArgumentException("Unknown fitness evaluator to be deserialized!") ;
            }
        }
    }
}
