﻿/* Uncomment following line to turn on debugging of this operator. */
//#define DEBUG_IVM

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MCScheduling.GeneticAlgorithm
{
    /**
     * The inversion mutation operator. It randomly chooses two points within
     * a chromosome with uniform probability, and reverses the genes within 
     * the range defined by these two points.
     * 
     * @author Tomas Tunys
     */
    public class CInversionMutation : CBaseMutationOperator
    {
        /**
         * Creates new inversion mutation operator for the specified configuration.
         * 
         * @param configuration The configuration of a genetic algorithm that uses
         *                      this operator.
         * @param mutationRate  The mutation rate.
         */
        public CInversionMutation(CConfiguration configuration, double mutationRate) : base(configuration, mutationRate)
        {
        }

        /**
         * Mutates the specified chromosome in a operator-specific way forming 
         * a new chromosome, which will then go to the next generation.
         * 
         * @param chromosome The chromosome to be mutated.
         */
        public override void apply(IChromosome chromosome)
        {
            if (shouldPerformMutation())
            {
                /* Randomly chooses two points within the chromosome. */
                int i = getConfiguration().getRandomGenerator().Next(chromosome.getSize()) ;
                int j = getConfiguration().getRandomGenerator().Next(chromosome.getSize()) ;

                if (i > j)
                {
                    int k = i ; i = j ; j = k ;
                }
#if DEBUG_IVM
                Console.WriteLine("Inversion Mutation: " + (i + 1) + " - " + (j + 1)) ;
                printOutGenome(chromosome, (int)Math.Ceiling(Math.Log10(chromosome.getSize())), true) ;
#endif
                /* Reverses the order of the genes within choses section. */
                while (i < j)
                {
                    swapGenes(chromosome, i++, j--) ;
                }
#if DEBUG_IVM
                printOutGenome(chromosome, (int)Math.Ceiling(Math.Log10(chromosome.getSize())), false) ;
#endif
            }
        }
    }
}
