﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MCScheduling.GeneticAlgorithm
{
    /**
     * Rank scaling may be used to prevent too quick convergence.
     * It work like this: the individuals in the specified population
     * are sorted according to their fitness - from the fittest to
     * the worst fit individual, and then each individual's fitness 
     * is assigned a new value based on its rank within the sorted 
     * population.
     */
    public class CRankScaling : IFitnessScaler
    {
        /**
         * Scales the fitness scores of the individuals 
         * within the specified population.
         * <p>
         * For more information look at the description 
         * of the RankScaling class.
         * </p>
         * 
         * @param population The population of individuals
         *                   to be scaled.
         */
        public void scale(CPopulation population)
        {
            if (!population.isSorted())
            {
                population.sortByFitness() ;
            }

            double rank = 0.0 ;

            for (int i = 0 ; i < population.getSize() ; i++)
            {
                population[i].setFitnessValue(++rank) ;
            }
        }
    }
}
