﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MCScheduling.GeneticAlgorithm
{
    /**
     * Defines a fitness calculation method.
     */
    public interface IFitnessEvaluator
    {
        /**
         * Determines the fitness of the specified chromosome. The fitness should be
         * then set for the chromosome using <em>IChromosome.setFitnessValue</em>.
         *
         * @param chromosome The chromosome whose fitness should be calculated.
         */
        void evaluate(IChromosome chromosome) ;

        /**
         * Determines the fitness of all chromosome in the specified population.
         * The fitness should be set for the chromosomes using 
         * <em>IChromosome.setFitnessValue</em>.
         *
         * @param population The population of chromosomes whose fitness 
         *                   should be calculated.
         */
        void evaluate(CPopulation population) ;
    }
}
