﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MCScheduling.GeneticAlgorithm
{
    /**
     * A pseudo-random number generator used by a genetic
     * algorithms.
     */
    public interface IRandomizer
    {
        /**
         * Returns a random non-negative 32-bit integer.
         * 
         * @return A random integer in the [0,2^32) interval.
         */
        int Next() ;

        /**
         * Returns a random non-negative integer lesser then the specified value.
         * 
         * @param maxValue The exclusive upper bound of the generated random integer.
         * 
         * @return A random integer from the [0,maxValue) interval.
         */
        int Next(int maxValue) ;

        /**
         * Returns a random integer bigger or equal to the specified minValue, and
         * lesser then the specified maxValue.
         * 
         * @param minValue The inclusive lower bound of the generated random integer.
         * @param maxValue The exclusive upper bound of the generated random integer.
         * 
         * @return A random integer from the [minValue,maxValue) interval.
         */
        int Next(int minValue, int maxValue) ;

        /**
         * Returns a random decimal number in the [0,1) interval.
         * 
         * @return A random decimal number in the [0,1) interval.
         */
        double NextDouble() ;
    }
}
