﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MCScheduling.GeneticAlgorithm
{
    /**
     * The selection operator is responsible for selecting an individual
     * chromosomes from the population that will be then used in the further
     * proceedings of the genetic algorithm: recombination phase, namely.
     */
    public interface ISelectionOperator
    {
        /**
         * Selects chromosomes from the population that will continue to the next
         * stage of the genetic algorithm: reproduction phase. This selection 
         * should be guided by the fitness values of the chromosomes, but that 
         * is up to the implementation of the selector.
         * 
         * @param population    The population the chromosome will be selected from.
         * @param matingPool    The mating pool the chromosome will be inserted into.
         * @param toSelectCount The total number of chromosomes to select.
         */
        void select(CPopulation population, CPopulation matingPool, int toSelectCount) ;
    }
}
