﻿using System ;
using System.Linq ;
using System.Text ;
using System.Collections.Generic ;
using MCScheduling.GeneticAlgorithm ;

namespace MCScheduling.MixedCriticality.GA
{
    /**
     * The CMixedCriticalityConfiguration represents a configuration of a genetic
     * algorithm for solving Mixed-criticality Scheduling Problems.
     * 
     * @author Tomas Tunys
     */
    public class CMixedCriticalityConfiguration : CConfiguration
    {
        /**
         * The mixed-criticality instance registered 
         * with this configuration.
         */
        CMixedCriticalityInstance m_instance ;

        /**
         * Creates a mixed-criticality configuration with the specified name.
         * 
         * @param configurationName The user-specific name for the configuration.
         */
        public CMixedCriticalityConfiguration(string configurationName) : base(configurationName)
        {
        }

        /**
         * Reads in the configuration from the specified file.
         *
         * @param configurationFile The file from which the configuration
         *                          will be created.
         * @param ignored           An ignored parameter.
         */
        public CMixedCriticalityConfiguration(string configurationFile, int ignored) : base(configurationFile, ignored)
        {
        }

        /**
         * Registeres with this configuration a mixed-criticality instance
         * that is going to be solved by the genetic algorithm that uses
         * this configuration.
         */
        public void setInstanceToSolve(CMixedCriticalityInstance instance)
        {
            m_instance = instance ;

            /* The chromosome size equals to the number of jobs in the instance. */
            setChromosomeSize(instance.getJobCount()) ;
        }

        /**
         * Returns the mixed-criticality instance that is going to be solved 
         * by the genetic algorithm that uses this configuration.
         */
        public CMixedCriticalityInstance getInstanceToSolve()
        {
            return m_instance ;
        }
    }
}
