﻿namespace MCScheduling.SimulatedAnnealing
{
    /**
     * The interface of the "probability distribution" of a state
     * being accepted according to its distance from the state
     * from which it has been created.
     * 
     * @author Tomas Tunys
     */
    public interface IAcceptor
    {
        /**
         * Returns the probability that a state that 
         * has the specified distance from a state
         * it has been created from; is going to be
         * accepted, or not.
         * 
         * @param distance The distance of two
         *                 neighbouring states.
         *                 
         * @return A number from 0.0 to 1.0 representing
         *         the probability of a state being accepted.
         */
        double Probability(double distance) ;
    }
}
